Package: ooplah
Title: Helper Functions for Class Object-Oriented Programming
Version: 0.2.0
Authors@R:
    person(given = "Raphael",
           family = "Sonabend",
           role = c("aut", "cre"),
           email = "raphaelsonabend@gmail.com",
           comment = c(ORCID = "0000-0001-9225-4654"))
Description: Helper functions for coding object-oriented programming with
    a focus on R6. Includes functions for assertions and testing, looping,
    and re-usable design patterns including Abstract and Decorator
    classes.
License: MIT + file LICENSE
URL: https://xoopR.github.io/ooplah/, https://github.com/xoopR/ooplah
BugReports: https://github.com/xoopR/ooplah/issues
Imports:
    R6
Suggests:
    devtools,
    testthat,
    withr
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
Roxygen: list(markdown = TRUE, r6 = TRUE)
RoxygenNote: 7.1.1
SystemRequirements: C++11
