## Test Results

No  WARNINGs, or ERRORs.

NOTEs:

* New submission
* Possibly unsafe calls - unlockBinding. These are required for the function
  which is specifically designed to modify the bindings of given environments

## Test environments

* Windows, R Release
* Mac OS X, R Release
* Ubuntu, R Release
* Ubuntu, R Oldrel
* Ubuntu, R Devel

## Reverse Dependencies

All reverse dependencies OK.