% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ooplah.R
\name{ooplah}
\alias{ooplah}
\title{R6 Class for testing and examples}
\description{
R6 Class for testing and examples
}
